//! # netpong
//! This is pretty silly and only let's your client send "ping"
//! over a connection, while letting a server reply pong to
//! every connection. But I had to make it for some reason.

use libpt::log::*;

use clap::Parser;
use clap_verbosity_flag::{InfoLevel, Verbosity};

mod client;
mod common;
mod server;

use common::Mode;

/// short about section displayed in help
const ABOUT_ROOT: &'static str = r##"
Let your hosts play ping pong over the network
"##;
/// longer about section displayed in help, is combined with [the short help](ABOUT_ROOT)
static LONG_ABOUT_ROOT: &'static str = r##"

    Connect to a ping pong server and periodically send a ping. The server will reply with a pong.
    That's it really. You can also host your own netpong server (server feature).
"##;

#[derive(Debug, Clone, Parser)]
#[command(
    author,
    version,
    about = ABOUT_ROOT,
    long_about = format!("{}{}", ABOUT_ROOT ,LONG_ABOUT_ROOT),
    help_template =
r#"{about-section}
{usage-heading} {usage}
{all-args}{tab}

libpt: {version}
Author: {author-with-newline}
"#
    )]
pub(crate) struct Cli {
    // clap_verbosity_flag seems to make this a global option implicitly
    /// set a verbosity, multiple allowed (f.e. -vvv)
    #[command(flatten)]
    pub(crate) verbose: Verbosity<InfoLevel>,

    /// show additional logging meta data
    #[arg(long)]
    pub(crate) meta: bool,

    #[cfg(feature = "server")]
    #[arg(short, long, default_value_t = false)]
    pub(crate) server: bool,

    #[arg(short, long, default_value_t = Mode::Tcp, ignore_case = true)]
    pub(crate) mode: Mode,

    /// Address of the server
    pub(crate) addr: std::net::SocketAddr,
}

fn cli_parse() -> Cli {
    let cli = Cli::parse();
    let ll: Level = match cli.verbose.log_level().unwrap().as_str() {
        "TRACE" => Level::TRACE,
        "DEBUG" => Level::DEBUG,
        "INFO" => Level::INFO,
        "WARN" => Level::WARN,
        "ERROR" => Level::ERROR,
        _ => {
            unreachable!();
        }
    };
    if cli.meta {
        Logger::init(None, Some(ll)).expect("could not initialize Logger");
    } else {
        // less verbose version
        Logger::init_mini(Some(ll)).expect("could not initialize Logger");
    }
    return cli;
}

fn main() {
    let cli = cli_parse();
    debug!("dumping cli args:\n{:#?}", cli);
    #[cfg(feature = "server")]
    if cli.server {
        info!("starting server");

        return;
    }
    // implicit else, so we can work without the server feature
    info!("starting client");
}
